@@ -60,7 +60,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 #define O_NOFOLLOW 0
 #endif
 
-static char swappath[PATH_MAX] = VARPREFIX"/var/lib/swapspace";
+static char swappath[PATH_MAX] = VARPREFIX"/lib/swapspace";
 static size_t swappath_len;
 
 
@@ -232,7 +232,16 @@ static struct Swapfile swapfiles[MAX_SWAPFILES+1];
 /// Have we been able to verify that /proc/swaps is in the expected format?
 static bool proc_swaps_read_ok = false;
 
+/// Can we allocate swapfiles using posix_allocate on this filesystem?
+static bool pfalloc_ok = true;
 
+/* Store filesystem type so we don't check everytime we allocate a swapfile
+ * __fsword_t is a glibc-internal type, but the docs claim unsigned int is fine
+ */
+static unsigned int fstype = 0;
+
+/// Does the filesystem support the NOCOW attribute?
+static bool nocow_ok = true;
 
 /// Print status information to stdout
 void dump_stats(void)
@@ -320,7 +329,6 @@ memsize_t swapfs_size(void)
   return fs_size(fsinfo.f_blocks, fsinfo.f_bsize);
 }
 
-
 /// Turn an existing file into an active swap.  Clobbers localbuf.
 /**
  * @return Whether file was activated successfully.
@@ -334,8 +342,6 @@ static bool enable_swapfile(const char file[])
   return ok;
 }
 
-
-
 /// Write arbitrary data to swapfile.  Clobbers localbuf.
 /** Fill a swapfile with the specified number of bytes of data.  We don't care
  * what the data is.
@@ -359,7 +365,7 @@ static memsize_t write_data(int fd, memsize_t bytes, bool persevere)
   bytes = ext_to_page(bytes);
 #ifdef HAVE_PFALLOCATE
  // Have posix_fallocate().  Much faster way of getting the file populated.
- if (!zero && posix_fallocate(fd, 0, bytes) == 0) return bytes;
+ if (pfalloc_ok && posix_fallocate(fd, 0, bytes) == 0) return bytes;
  // We have no error backchannel here, so on failure, try the old way.
 #endif
   /* Zero buffer before using it to write data to swapfile.  This doesn't do
@@ -437,17 +443,29 @@ int set_no_cow(int fd)
   int err;
 
   err = ioctl(fd, FS_IOC_GETFLAGS, &attr);
-  if (err != 0) return err;
+  if (err != 0)
+  {
+    if (errno == ENOTSUP)
+    {
+      nocow_ok = false;
+      logm(LOG_NOTICE, "Filesystem does not support attributes!");
+    }
+    else return err;
+  }
   attr |= FS_NOCOW_FL;
   err = ioctl(fd, FS_IOC_SETFLAGS, &attr);
-  if (err != 0) return err;
+  if (err != 0)
+  {
+    if (errno == ENOTSUP)
+    {
+      nocow_ok = false;
+      logm(LOG_NOTICE, "NOCOW attribute not supported!");
+    }
+    else return err;
+  }
   return 0;
 }
 
-/* Store filesystem type so we don't check everytime we allocate a swapfile
- * __fsword_t is a glibc-internal type, but the docs claim unsigned int is fine
- */
-unsigned int fstype = 0;
 
 /// Handle filesystem-specific operations needed for swapfiles
 /* Some filesystems, like BTRFS, require special operations to be performed on
@@ -467,7 +485,7 @@ void specialfs(const char path[])
     if (err == 0) fstype = buf.f_type;
     else log_perr_str(LOG_WARNING, "Could not detect filesystem", path, errno);
   }
-  else if (fstype == BTRFS_SUPER_MAGIC)
+  if (fstype == BTRFS_SUPER_MAGIC)
   {
     err = runcommandformat("%s property set '%s' compression none", "btrfs", path);
     if (err != 0) logm(LOG_WARNING, "Could not disable BTRFS compression! Return Code: %d", err);
@@ -501,8 +519,11 @@ static memsize_t make_swapfile(const char file[], memsize_t size)
     return 0;
   }
 
-  int err = set_no_cow(fd);
-  if (unlikely(err != 0)) log_perr_str(LOG_WARNING, "Could not disable COW", file, errno);
+  if (nocow_ok)
+  {
+    int err = set_no_cow(fd);
+    if (unlikely(err != 0)) log_perr_str(LOG_WARNING, "Could not disable COW", file, errno);
+  }
 
   specialfs(file);
 
@@ -864,8 +885,20 @@ bool alloc_swapfile(memsize_t size)
   if (unlikely(!enable_swapfile(file)))
   {
     unlink(file);
-    request_diet();
-    return false;
+    swapfiles[newswap].size = 0;
+    if (pfalloc_ok && errno == EINVAL)
+    {
+      logm(LOG_NOTICE, "Quick swapfile creation disabled.");
+      // If we get EINVAL, then we can't actually use posix_fallocate
+      pfalloc_ok = false;
+      // Try again
+      return alloc_swapfile(size-2*getpagesize());
+    }
+    else
+    {
+      request_diet();
+      return false;
+    }
   }
 
   sequence_number = inc_swapno(sequence_number);
